package
Version:
v0.0.0-...-4d1c14d
Opens a new window with list of versions in this module.
Published: May 29, 2026
License: MPL-2.0
Opens a new window with license information.
Imports: 6
Opens a new window with list of imports.
Imported by: 0
Opens a new window with list of known importers.
Documentation
¶
func DevString() (short, long string)
func IsMissingMetadata() bool
Source Files
¶
Click to show internal directories.
Click to hide internal directories.